import datetime as dt
from dateutil.relativedelta import relativedelta


def start(last=False):
    """
    Returns the date of the first day of the week.
    Will use the current week by default. Use last=True to get the first
    date of last week.
    """
    today = dt.date.today()
    day_of_week = today.weekday()  # gets the number of the current day
    start_date = today - relativedelta(days=day_of_week)
    if last:
        return start_date - relativedelta(days=7)
    else:
        return start_date


def end(last=False):
    """
    Returns the date of the last day of the week.
    Uses the current week by default. Use last=True to get the last
    date of the previous week.
    """
    first_day = start(last=last)
    return first_day + relativedelta(days=6)


def days_MS():
    return ['Monday','Tuesday','Wednesday','Thursday','Friday','Saturday', 'Sunday']


# LAYOUT
mint = '#48CFAD'
aqua = '#4FC1E9'
lavender = '#AC92EC'
bluejeans = '#5D9CEC'
sunflower = '#FFCE54'
grapefruit = '#ED5565'
bittersweet = '#FC6E51'
grass = '#A0D468'
pink_rose = '#EC87C0'
light_grey = '#F5F7FA'
mid_grey = '#CCD1DC'
dark_grey = '#656D78'



from io import StringIO
import pandas as pd


def getData(service, csv_filename, g, dates=False, usecols=None):
    repo = g.get_repo('losthippo/tps-data')
    file = f"{service}/data/{csv_filename}"
    f = repo.get_contents(file).decoded_content
    s = str(f, 'utf-8')
    data = StringIO(s)
    return pd.read_csv(data, dayfirst=True, parse_dates=dates, usecols=usecols)


import datetime as dt
from dateutil.relativedelta import relativedelta


def current_quarter():
    """
    Returns the number of the current quarter
    """
    quarters = [4,4,4,1,1,1,2,2,2,3,3,3]
    return quarters[dt.date.today().month]


def start(last=False):
    """
    Returns the first date of the current quarter by default.
    Use last=True to get the first date of the last quarter.
    """
    quarter = 4

    year = dt.date.today().year

    starts = {1: 4, 2: 7, 3: 10, 4: 1}
    if last == False:
        quarter = current_quarter()
    else:
        if current_quarter() > 1:
            quarter == current_quarter() - 1
        else:
            pass

    return dt.date(year, starts[quarter], 1)


def end(last=False):
    """
    Returns the last date of the current quarter by default.
    Use last=True to get the last date of the previous quarter.
    """
    start_date = start(last=last)
    return start_date + relativedelta(months=3, days=-1)




import plotly.graph_objects as go
import plotly.express as px

margins = go.layout.Margin(l=75, r=50, b=50, t=25)
margins_sm = go.layout.Margin(l=20, r=25, b=25, t=25)
margins_none = go.layout.Margin(l=0, r=0, b=0, t=0)


def fig_empty():
    empty = go.Figure()
    empty.update_layout(
        autosize=False,
        height=275,
        annotations=[
            dict(
                x=0.5,
                y=0.5,
                text='None in this period',
                xref='paper',
                yref='paper',
                showarrow=False,
                font={'size': 28}
            ),
        ],
        xaxis=dict(
            title="",
            zeroline=False,
            showticklabels=False,
        ),
        yaxis=dict(
            title="",
            zeroline=False,
            showticklabels=False,
        ),
    )
    return empty


def no_data_in_period_check(fig):
    if len(fig.data) > 0:
        pass
    else:
        fig = fig_empty()
    return fig


def horizontal_bars(data, x_data, y_data, label_data, height=275, category_order='total ascending', group=False, color=None, hover_data=None, hovermode=True, title=None):
    fig = px.bar(data, y=y_data, x=x_data, orientation='h', text=label_data, height=height, color=color, hover_data=hover_data, title=title)
    fig.update_xaxes(title="", showticklabels=False)
    fig.update_yaxes(title="", showticklabels=True, categoryorder=category_order)
    fig.update_layout(margin=margins)
    if group:
        fig.update_layout(barmode='group')
    if hovermode == False:
        fig.update_layout(hovermode=False)
    return no_data_in_period_check(fig)


def vertical_bars(data, x_data, y_data, label_data, height=None, color=None, category_order='trace', group=False, hover_data=None, hovermode=True, title=None):
    fig = px.bar(data, y=y_data, x=x_data, text=label_data, height=height, color=color, hover_data=hover_data, title=title)
    fig.update_xaxes(title="", showticklabels=True, categoryorder=category_order)
    fig.update_yaxes(title="", showticklabels=False)
    fig.update_layout(margin=margins)
    if group:
        fig.update_layout(barmode='group')
    if hovermode==False:
        fig.update_layout(hovermode=False)
    return no_data_in_period_check(fig)


def pie_chart(data, values, names, hole=0.5):
    fig = px.pie(data, values=values, names=names, hole=hole)
    return no_data_in_period_check(fig)



import datetime as dt
from dateutil.relativedelta import relativedelta


def start(last=False):
    """
    Returns the first date of the current month by default.
    Use last=True to get the first date of last month.
    """
    today = dt.date.today()
    start_date = dt.date(today.year, today.month, 1)
    if last:
        return start_date + relativedelta(months=-1)
    return start_date


def end(last=False):
    """
        Returns the last date of the current month by default.
        Use last=True to get the last date of the previous month.
        """
    return start(last=last) + relativedelta(months=1, days=-1)



from textwrap import fill
from dash import html
from io import StringIO
import pandas as pd
import numpy as np
import json


def strip_whitespace(df):
    df_obj = df.select_dtypes(['object'])
    df[df_obj.columns] = df_obj.apply(lambda x: x.str.strip())
    return df


def make_dash_table(df):
    """ Return a dash definition of an HTML table for a Pandas dataframe """
    table = []
    for index, row in df.iterrows():
        html_row = []
        for i in range(len(row)):
            html_row.append(html.Td([row[i]]))
        table.append(html.Tr(html_row))
    return table


def age_bins(df, col):
    bins = pd.cut(df[col], [0, 24, 34, 44, 54, 64, np.inf], labels=['16-24', '25-34', '35-44', '45-54', '55-64', '65+'])
    return bins


def label(df, numerator, denominator=False):
    """ Function that takes a dataframe and a column of values and \
    generates a percentage that each value represents. \
    It also then combines these into a label that can be used for charting.

    Denominator can be specified if the calculation should be as a percentage
    of a particular variable, e.g. number of clients that were worked with in a period
    instead of the total in the dataframe
    """
    if denominator:
        df['P'] = round(df[numerator] * 100 / denominator, 2)
    else:
        df['P'] = round(df[numerator] * 100 / df[numerator].sum(), 2)
    df['Label'] = df[numerator].map(str) + " (" + df['P'].map(str) + "%)"
    return df


def business_minutes(start, end, biz_hours=pd.offsets.BusinessHour()):
    times_array = pd.date_range(start, end, freq='min')
    times = pd.DataFrame(times_array, columns=['timestamp'])
    times['IsBizMin'] = times['timestamp'].apply(pd.Timestamp).apply(biz_hours.is_on_offset)
    
    biz_min = times.IsBizMin.sum() 
    
    if biz_min != 0:
        return biz_min -1
    
    else:
        return biz_min


def non_business_minutes(start, end, biz_hours=pd.offsets.BusinessHour()):
    times_array = pd.date_range(start, end, freq='min')
    times = pd.DataFrame(times_array, columns=['timestamp'])
    times['IsBizMin'] = times['timestamp'].apply(pd.Timestamp).apply(biz_hours.is_on_offset)
    
    non_biz_min = len(times) - times.IsBizMin.sum() 
    
    if non_biz_min != 0:
        return non_biz_min -1
    
    else:
        return non_biz_min


def calculate_outside_hours(df, start, end):
    biz_hours = pd.offsets.BusinessHour()
    in_hours = []
    outside_hours = []

    for index, row in df.iterrows():
        # Create an array of minutes that the start and end time covers
        times_array = pd.date_range(start=row[start],
                                    end=row[end],
                                    freq='min')
        # Create a df from the array
        times = pd.DataFrame(times_array, columns=['timestamp'])

        # Check whether each minute is in business time or not
        times['IsBizHour'] = times['timestamp'].apply(pd.Timestamp).apply(biz_hours.is_on_offset)

        # Create an array of the number of hours outside business hours
        outside_hours.append((len(times) - times.IsBizHour.sum()) / 60)

    # Sum the hours outside and return the number
    return sum(outside_hours)


def load_callback_data(datasets, data):
    my_data = json.loads(datasets)
    return pd.read_json(my_data[data], orient="split", convert_dates=True)


def create_csv(name_string, datasets, data, start_date, end_date, columns_to_include=False, columns_to_ignore=False):
    filename = f"{name_string} - {str(start_date)} to {str(end_date)}.csv"
    s = StringIO()

    df = load_callback_data(datasets, data)
    if columns_to_include:
        df = df[columns_to_include].reset_index(drop=True)
    if columns_to_ignore:
        df = df.loc[:, ~df.columns.isin(columns_to_ignore)].reset_index(drop=True)

    df.to_csv(s, index=False)
    content = s.getvalue()
    return dict(filename=filename, content=content, type="text/csv")


def group_count_and_label(data, group_by, aggregation={'POEID':'count'}, index='POEID', apply_label=True, simple=False, n=5, denominator=False, dropna=True, reindex=False):
    df = data.groupby(group_by, dropna=dropna).agg(aggregation).reset_index()
    
    if reindex:
        df = data.groupby(group_by, dropna=dropna).agg(aggregation).reindex(reindex, fill_value=0).reset_index()
    else:
        pass
    
    if simple:
        df = df.replace(df.groupby(group_by, dropna=dropna).sum().sort_values(index, ascending=False).index[n:], 'Other') \
            .groupby(group_by).sum().reset_index()
    else:
        pass

    if apply_label:
        df = label(df=df, numerator=index, denominator=denominator)
    else:
        pass

    return df


def get_engagement_types(df, column, startswith):
    dff = df[df[column].str.startswith(startswith)]
    dff[column] = dff[column].str.replace(startswith, '')
    return dff.reset_index(drop=True)


def group_by_week(df, date_column, count_column='POEID'):
    return df.groupby(pd.Grouper(key=date_column, freq='W-MON'))[[count_column]].count().reset_index()



from dateutil.relativedelta import relativedelta
from tps_dashboard_utils.date_functions import monthly


def start(last=False):
    """
    Used to get 6 whole months of data. Uses the current month-6 by default
    but will use the last month - 6 if last=True
    """
    start_date = monthly.start(last=last)
    return start_date + relativedelta(months=-6)


def six_month_end(last=False):
    """
    Used to get 6 whole months of data. Uses the current month-6 by default
    but will use the last month - 6 if last=True
    """
    start_date = start(last=last)
    return start_date + relativedelta(months=6, days=-1)



from tps_dashboard_utils.date_functions import monthly
from tps_dashboard_utils.date_functions import quarterly
from tps_dashboard_utils.date_functions import six_monthly
from tps_dashboard_utils.date_functions import weekly
from tps_dashboard_utils.date_functions import yearly


from dash import dcc
from tps_dashboard_utils.colours import categorical
import dash_bootstrap_components as dbc
import plotly.io as pio

pio.templates.default = "plotly_white"
pio.templates['plotly_white'].layout.colorway = categorical.pastel
pio.templates['plotly_white'].layout.xaxis.gridcolor = 'white'
pio.templates['plotly_white'].layout.yaxis.gridcolor = 'white'


def Dates(earliest, latest):
    date_picker = dcc.DatePickerRange(
        id='date-range',
        start_date=earliest,
        end_date=latest,
        display_format='DD-MM-YYYY',
        persistence=True,
        persisted_props=['start_date', 'end_date'],
        persistence_type='session'
    )
    return date_picker


def Navbar(service):
    navbar = dbc.NavbarSimple(
        children=[
            # dbc.NavItem(dbc.NavLink("Data refreshed: " + lastUpdate)),
            # dbc.NavItem(dbc.NavLink("Page2", href="/Page2", external_link=True)),
        ],
        brand=service,
        brand_href="/home",
        sticky="top",
        color="primary",
        dark=True,
        expand='lg',
    )
    return navbar



import datetime as dt
from dateutil.relativedelta import relativedelta


def year_start(last=False):
    """
    Returns the first date of the current year by default.
    Use last=True to get the first date of the last year.
    """
    this_year = dt.date(dt.date.today().year, 1, 1)
    if last:
        return this_year + relativedelta(years=-1)
    else:
        return this_year


def year_end(last=False):
    """
    Returns the last date of the current year by default.
    Use last=True to get the last date of the previous year.
    """
    start_date = year_start(last=last)
    return start_date + relativedelta(years=1, days=-1)



from tps_dashboard_utils.colours import colour_names as cn

pastel = ['#86E3CE', '#D0E6A5', '#FFDD94', '#FA897B', '#CCABDB']
rag = ['#FF7588', '#FFA87D', '#16D39A']
flat = [cn.mint, cn.aqua, cn.lavender, cn.bluejeans, cn.pink_rose, cn.grapefruit, cn.bittersweet, cn.sunflower,
        cn.grass, cn.light_grey, cn.mid_grey, cn.dark_grey]